@@ -5,10 +5,13 @@ the debuggee's address space.
 
 from __future__ import annotations
 
+import re
 import string
+from typing import Iterator
 
 import pwndbg
 import pwndbg.aglib.memory
+from pwndbg.lib.memory import Page
 
 length = 15
 
@@ -51,3 +54,16 @@ def get(address: int, maxlen: int | None = None, maxread: int | None = None) ->
         return sz
 
     return sz[:maxlen] + "..."
+
+
+def yield_in_page(page: Page, n=4) -> Iterator[str]:
+    """Yields strings of length >= n found in a given vmmap page"""
+    try:
+        data = pwndbg.aglib.memory.read(addr=page.vaddr, count=page.memsz, partial=True)
+    except pwndbg.dbg_mod.Error:
+        # E.g. we cannot read [vvar] page even though it has a READ permission
+        return
+
+    for match in re.finditer(rb"[ -~]{%d,}" % n, data):
+        decoded_str = match.group().decode("ascii", errors="ignore")
+        yield decoded_str
