@@ -2,10 +2,11 @@ from __future__ import annotations
 
 import argparse
 
+import pwndbg.aglib.file
 import pwndbg.color
 import pwndbg.commands
-import pwndbg.gdblib.file
 import pwndbg.wrappers.checksec
+from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(
     description="Prints out the binary security settings using `checksec`."
@@ -14,8 +15,8 @@ parser = argparse.ArgumentParser(
 parser.add_argument("-f", "--file", type=str, help="Specify the file to run `checksec` on.")
 
 
-@pwndbg.commands.ArgparsedCommand(parser, command_name="checksec")
+@pwndbg.commands.ArgparsedCommand(parser, command_name="checksec", category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWithFile
 def checksec(file: str) -> None:
-    local_path = file or pwndbg.gdblib.file.get_proc_exe_file()
+    local_path = file or pwndbg.aglib.file.get_proc_exe_file()
     print(pwndbg.wrappers.checksec.get_raw_out(local_path))
