@@ -2,18 +2,29 @@ from __future__ import annotations
 
 import argparse
 import ast
+import functools
+import logging
 import os
 import sys
 from collections import defaultdict
 from typing import Any
+from typing import Callable
 from typing import DefaultDict
 from typing import Dict
 from typing import List
+from typing import Optional
 from typing import Tuple
+from typing import TypeVar
 
-import gdb
+import unicorn as U
+from typing_extensions import ParamSpec
 
 import pwndbg
+import pwndbg.aglib.arch
+import pwndbg.aglib.disasm
+import pwndbg.aglib.nearpc
+import pwndbg.aglib.regs
+import pwndbg.aglib.symbol
 import pwndbg.arguments
 import pwndbg.chain
 import pwndbg.color
@@ -22,21 +33,26 @@ import pwndbg.color.memory as M
 import pwndbg.color.syntax_highlight as H
 import pwndbg.commands
 import pwndbg.commands.telescope
-import pwndbg.gdblib.disasm
-import pwndbg.gdblib.heap_tracking
-import pwndbg.gdblib.nearpc
-import pwndbg.gdblib.regs
-import pwndbg.gdblib.symbol
-import pwndbg.gdblib.vmmap
-import pwndbg.ghidra
 import pwndbg.integration
 import pwndbg.ui
+from pwndbg.aglib.arch import get_thumb_mode_string
 from pwndbg.color import ColorConfig
 from pwndbg.color import ColorParamSpec
 from pwndbg.color import message
 from pwndbg.color import theme
 from pwndbg.commands import CommandCategory
-from pwndbg.gdblib.arch import get_thumb_mode_string
+
+if pwndbg.dbg.is_gdblib_available():
+    import gdb
+
+    import pwndbg.gdblib.ptmalloc2_tracking
+    import pwndbg.gdblib.symbol
+    import pwndbg.ghidra
+
+log = logging.getLogger(__name__)
+
+T = TypeVar("T")
+P = ParamSpec("P")
 
 theme.add_param("backtrace-prefix", "►", "prefix for current backtrace label")
 
@@ -75,6 +91,67 @@ def clear_screen(out=sys.stdout) -> None:
     out.write("\x1b[H\x1b[2J")
 
 
+config_reserve_lines = pwndbg.config.add_param(
+    "context-reserve-lines",
+    "if-ctx-fits",
+    "when to reserve lines after the prompt to reduce context shake",
+    help_docstring="""
+The "if-ctx-fits" setting only reserves lines if the whole context would still fit vertically in the current terminal window.
+It doesn't take into account line-wrapping due to insufficient terminal width.
+""",  # TODO: maybe it could take into account line-wrapping?
+    param_class=pwndbg.lib.config.PARAM_ENUM,
+    enum_sequence=["never", "if-ctx-fits", "always"],
+)
+
+
+def reserve_lines_maybe(cmd_lines: int) -> None:
+    """
+    Scroll the terminal up a few lines to reduce shaking
+    when repeatedly printing the context.
+
+    Only do this if the context would still fit on
+    the screen.
+    """
+    if config_reserve_lines == "never":
+        return
+
+    # If we are clearing the screen, we already have the maximum number of empty
+    # lines below us, no need to scroll.
+    if config_reserve_lines == "if-ctx-fits" and config_clear_screen:
+        return
+
+    # Could use .get_cmd_window_size() but lldb doesn't provide it
+    rows, _ = pwndbg.ui.get_window_size(sys.stdout)
+    leftover_lines = rows - cmd_lines
+    # Take away one line for the prompt (e.g. pwndbg>) which is printed after the context
+    leftover_lines -= 1
+
+    # Need one line for the newline generated by pressing enter
+    needed_empty = 1
+    # Also need to cover extra lines that are printed before we start calculating/printing
+    # the context.
+    # Even though they are cleared when config_clear_screen=True, they still
+    # cause shake, so we still account for them.
+    needed_empty += pwndbg.dbg.pre_ctx_lines
+
+    if not config_clear_screen:
+        # Since we are not clearing the screen, we assume the prompt is at the bottom
+        # of the window
+        if config_reserve_lines == "always" or leftover_lines >= needed_empty:
+            # We always write to stdout since that is where the prompt will be
+            # ANSII codes
+            # \x1b[#S  scroll the terminal up by # lines
+            # \x1b[#A  move the cursor up by # lines
+            sys.stdout.write(f"\x1b[{needed_empty}S\x1b[{needed_empty}A")
+    else:
+        assert config_reserve_lines == "always"
+        # Since we are clearing the screen, leftover_lines describes actual empty lines.
+        leftover_lines = max(leftover_lines, 0)
+        if leftover_lines < needed_empty:
+            to_scroll = needed_empty - leftover_lines
+            sys.stdout.write(f"\x1b[{to_scroll}S\x1b[{to_scroll}A")
+
+
 config_clear_screen = pwndbg.config.add_param(
     "context-clear-screen", False, "whether to clear the screen before printing the context"
 )
@@ -83,7 +160,7 @@ config_output = pwndbg.config.add_param(
 )
 config_context_sections = pwndbg.config.add_param(
     "context-sections",
-    "regs disasm code ghidra stack backtrace expressions threads heap-tracker",
+    "regs disasm code ghidra stack backtrace expressions threads heap_tracker",
     "which context sections are displayed (controls order)",
 )
 config_max_threads_display = pwndbg.config.add_param(
@@ -94,7 +171,7 @@ config_max_threads_display = pwndbg.config.add_param(
 
 # Storing output configuration per section
 outputs: Dict[str, str] = {}
-output_settings = {}
+output_settings: DefaultDict[str, Dict[str, Any]] = defaultdict(dict)
 
 
 @pwndbg.config.trigger(config_context_sections)
@@ -188,6 +265,9 @@ class CallOutput:
     def write(self, data) -> None:
         self.func(data)
 
+    def writelines(self, lines_iterable) -> None:
+        self.func("".join(lines_iterable))
+
     def flush(self):
         try:
             return self.func.flush()
@@ -216,7 +296,7 @@ parser = argparse.ArgumentParser(description="Sets the output of a context secti
 parser.add_argument(
     "section",
     type=str,
-    help="The section which is to be configured. ('regs', 'disasm', 'code', 'stack', 'backtrace', and/or 'args')",
+    help="The section which is to be configured. ('regs', 'disasm', 'code', 'stack', 'backtrace', 'ghidra', 'args', 'threads', 'heap_tracker', 'expressions', and/or 'last_signal')",
 )
 parser.add_argument("path", type=str, help="The path to which the output is written")
 parser.add_argument("clearing", type=bool, help="Indicates weather to clear the output")
@@ -240,16 +320,222 @@ parser.add_argument(
 def contextoutput(section, path, clearing, banner="both", width: int = None):
     if not banner:  # synonym for splitmind backwards compatibility
         banner = "none"
-    if banner not in ("both", "top", "bottom", "none"):
+    elif banner not in ("both", "top", "bottom", "none"):
         raise argparse.ArgumentError(banner_arg, f"banner can not be '{banner}'")
 
     outputs[section] = path
-    output_settings[section] = {
-        "clearing": clearing,
-        "width": width,
-        "banner_top": banner in ["both", "top"],
-        "banner_bottom": banner in ["both", "bottom"],
-    }
+    output_settings[section].update(
+        {
+            "clearing": clearing,
+            "width": width,
+            "banner_top": banner in ("both", "top"),
+            "banner_bottom": banner in ("both", "bottom"),
+        }
+    )
+
+
+def resetcontextoutput(section):
+    target = outputs.pop(section, None)
+    if target:
+        # Remove all settings except for the ones that are not related to output redirection
+        output_settings[section] = {
+            k: v
+            for k, v in output_settings[section].items()
+            if k not in ("clearing", "width", "banner_top", "banner_bottom")
+        }
+
+
+# Context history
+context_history: DefaultDict[str, List[List[str]]] = defaultdict(list)
+selected_history_index: Optional[int] = None
+
+context_history_size = pwndbg.config.add_param(
+    "context-history-size", 50, "number of context history entries to store"
+)
+
+
+@pwndbg.config.trigger(context_history_size)
+def history_size_changed() -> None:
+    if context_history_size <= 0:
+        context_history.clear()
+    else:
+        for section in context_history:
+            context_history[section] = context_history[section][-int(context_history_size) :]
+
+
+def serve_context_history(function: Callable[P, List[str]]) -> Callable[P, List[str]]:
+    @functools.wraps(function)
+    def _serve_context_history(*a: P.args, **kw: P.kwargs) -> List[str]:
+        global selected_history_index
+        assert "context_" in function.__name__
+        section_name = function.__name__.replace("context_", "")
+
+        # If the history is disabled, just return the current output
+        if context_history_size <= 0:
+            return function(*a, **kw)
+
+        # Add the current section to the history if it is not already there
+        current_output = []
+        if pwndbg.aglib.proc.alive:
+            # Do not reevaluate the expressions section because its content is not deterministic.
+            # Instead, reuse the last evaluated expression and rely on the other sections to deselect
+            # the history entry if the output changed.
+            # https://github.com/pwndbg/pwndbg/issues/2579
+            if (
+                section_name == "expressions"
+                and selected_history_index is not None
+                and len(context_history[section_name]) > 0
+            ):
+                current_output = context_history[section_name][-1]
+            else:
+                current_output = function(*a, **kw)
+            if (
+                len(context_history[section_name]) == 0
+                or context_history[section_name][-1] != current_output
+            ):
+                context_history[section_name].append(current_output)
+                selected_history_index = None
+        # Show the history if the process is not running anymore
+        elif context_history[section_name] and selected_history_index is None:
+            selected_history_index = len(context_history[section_name]) - 1
+
+        # Truncate the history to the configured size
+        context_history[section_name] = context_history[section_name][-int(context_history_size) :]
+        history = context_history[section_name]
+
+        if selected_history_index is None:
+            return current_output or function(*a, **kw)
+        if not history or selected_history_index >= len(history):
+            return []
+        return history[selected_history_index]
+
+    return _serve_context_history
+
+
+def history_handle_unchanged_contents() -> None:
+    if not context_history:
+        return
+    longest_history = max(len(h) for h in context_history.values())
+    for section_name, history in context_history.items():
+        # Duplicate the last entry if it is the same as the previous one
+        # and wasn't added when the history was updated
+        if len(history) == longest_history - 1:
+            context_history[section_name].append(history[-1])
+        # Prepend empty entries to the history to make all sections have the same length
+        elif len(history) < longest_history - 1:
+            context_history[section_name] = [
+                [] for _ in range(longest_history - 1 - len(history))
+            ] + history
+
+
+parser = argparse.ArgumentParser(description="Select previous entry in context history.")
+parser.add_argument(
+    "count",
+    type=int,
+    nargs="?",
+    default=1,
+    help="The number of entries to go back in history",
+)
+
+
+@pwndbg.commands.ArgparsedCommand(parser, aliases=["ctxp"], category=CommandCategory.CONTEXT)
+def contextprev(count) -> None:
+    global selected_history_index
+    if not context_history:
+        print(message.error("No context history captured"))
+        return
+    if selected_history_index is None:
+        longest_history = max(len(h) for h in context_history.values())
+        new_index = longest_history - count - 1
+    else:
+        new_index = selected_history_index - count
+    selected_history_index = max(0, new_index)
+    context()
+
+
+parser = argparse.ArgumentParser(description="Select next entry in context history.")
+parser.add_argument(
+    "count",
+    type=int,
+    nargs="?",
+    default=1,
+    help="The number of entries to go forward in history",
+)
+
+
+@pwndbg.commands.ArgparsedCommand(parser, aliases=["ctxn"], category=CommandCategory.CONTEXT)
+def contextnext(count) -> None:
+    global selected_history_index
+    if not context_history:
+        print(message.error("No context history captured"))
+        return
+    longest_history = max(len(h) for h in context_history.values())
+    if selected_history_index is None:
+        new_index = longest_history - 1
+    else:
+        new_index = selected_history_index + count
+    selected_history_index = min(longest_history - 1, new_index)
+    context()
+
+
+parser = argparse.ArgumentParser(
+    description="Search for a string in the context history and select that entry."
+)
+parser.add_argument(
+    "needle",
+    type=str,
+    help="The string to search for in the context history",
+)
+parser.add_argument(
+    "section",
+    type=str,
+    nargs="?",
+    default=None,
+    help="The section to search in. If not provided, search in all sections",
+)
+
+
+@pwndbg.commands.ArgparsedCommand(parser, aliases=["ctxsearch"], category=CommandCategory.CONTEXT)
+def contextsearch(needle, section) -> None:
+    if not section:
+        sections = context_history.keys()
+    else:
+        if section not in context_history:
+            print(message.error(f"Section '{section}' not found in context history."))
+            return
+        sections = [section]
+
+    matches: List[Tuple[str, int]] = []
+    for section in sections:
+        for i, entry in enumerate(context_history[section]):
+            if not any(m[1] == i for m in matches) and any(needle in line for line in entry):
+                matches.append((section, i))
+    matches.sort(key=lambda m: m[1], reverse=True)
+
+    if not matches:
+        print(message.error(f"String '{needle}' not found in context history."))
+        return
+
+    # Select first match before currently selected entry
+    global selected_history_index
+    if selected_history_index is None:
+        next_match = matches[0]
+    else:
+        for match in matches:
+            if match[1] < selected_history_index:
+                next_match = match
+                break
+        else:
+            next_match = matches[0]
+            print(message.warn("No more matches before the current entry. Starting from the top."))
+
+    selected_history_index = next_match[1]
+    print(
+        message.info(
+            f"Found {len(matches)} match{'es' if len(matches) > 1 else ''}. Selected entry {next_match[1] + 1} for match in section '{next_match[0]}'."
+        )
+    )
+    context()
 
 
 # Watches
@@ -268,7 +554,7 @@ parser.add_argument(
     type=str,
     default="eval",
     nargs="?",
-    choices=["eval", "execute"],
+    choices=("eval", "execute"),
     help="""Command to be used with the expression.
 - eval: the expression is parsed and evaluated as in the debugged language.
 - execute: the expression is executed as a GDB command.""",
@@ -302,6 +588,7 @@ def contextunwatch(num) -> None:
     expressions.pop(int(num) - 1)
 
 
+@serve_context_history
 def context_expressions(target=sys.stdout, with_banner=True, width=None):
     if not expressions:
         return []
@@ -327,16 +614,27 @@ def context_expressions(target=sys.stdout, with_banner=True, width=None):
         if cmd == "execute":
             output.append(value)
 
+    # gdb can output single elements with \n's
+    # we split it up here so the cmd_line calculation is correct in context()
+    _output = []
+    for val in output:
+        _output.extend(val.split("\n"))
+    output = _output
+
     return banner + output if with_banner else output
 
 
 config_context_ghidra = pwndbg.config.add_param(
     "context-ghidra",
     "never",
-    "when to try to decompile the current function with ghidra (slow and requires radare2/r2pipe or rizin/rzpipe) (valid values: always, never, if-no-source)",
+    "when to try to decompile the current function with ghidra",
+    help_docstring="Doing this is slow and requires radare2/r2pipe or rizin/rzpipe.",
+    param_class=pwndbg.lib.config.PARAM_ENUM,
+    enum_sequence=["always", "never", "if-no-source"],
 )
 
 
+@serve_context_history
 def context_ghidra(target=sys.stdout, with_banner=True, width=None):
     """
     Print out the source of the current function decompiled by ghidra.
@@ -370,18 +668,36 @@ parser.add_argument(
     nargs="*",
     type=str,
     default=None,
-    help="Submenu to display: 'reg', 'disasm', 'code', 'stack', 'backtrace', 'ghidra', and/or 'args'",
+    help="Submenu to display: 'regs', 'disasm', 'code', 'stack', 'backtrace', 'ghidra', 'args', 'threads', 'heap_tracker', 'expressions', and/or 'last_signal'",
+)
+parser.add_argument(
+    "--on",
+    dest="enabled",
+    action="store_true",
+    default=None,
+    help="Show the section(s) in subsequent context commands again. The section(s) have to be in the 'context-sections' list.",
+)
+parser.add_argument(
+    "--off",
+    dest="enabled",
+    action="store_false",
+    default=None,
+    help="Do not show the section(s) in subsequent context commands even though they might be in the 'context-sections' list.",
 )
 
 
 @pwndbg.commands.ArgparsedCommand(parser, aliases=["ctx"], category=CommandCategory.CONTEXT)
-@pwndbg.commands.OnlyWhenRunning
-def context(subcontext=None) -> None:
+def context(subcontext=None, enabled=None) -> None:
     """
     Print out the current register, instruction, and stack context.
 
-    Accepts subcommands 'reg', 'disasm', 'code', 'stack', 'backtrace', 'ghidra' and 'args'.
+    Accepts subcommands 'reg', 'disasm', 'code', 'stack', 'backtrace', 'ghidra', 'args', 'threads', 'heap_tracker', 'expressions', and/or 'last_signal'.
     """
+    # Allow to view history after the program has exited
+    if not pwndbg.aglib.proc.alive and (context_history_size <= 0 or not context_history):
+        log.error("context: The program is not being run.")
+        return None
+
     if subcontext is None:
         subcontext = []
     args = subcontext
@@ -389,7 +705,15 @@ def context(subcontext=None) -> None:
     if len(args) == 0:
         args = config_context_sections.split()
 
-    sections = [("legend", lambda *args, **kwargs: [M.legend()])] if args else []
+    sections = []
+    if args:
+        if selected_history_index is None:
+            sections.append(("legend", lambda *args, **kwargs: [M.legend()]))
+        else:
+            longest_history = max(len(h) for h in context_history.values())
+            history_status = f" (history {selected_history_index + 1}/{longest_history})"
+            sections.append(("legend", lambda *args, **kwargs: [M.legend() + history_status]))
+
     sections += [(arg, context_sections.get(arg[0], None)) for arg in args]
 
     result = defaultdict(list)
@@ -398,16 +722,21 @@ def context(subcontext=None) -> None:
         if func:
             target = output(section)
             # Last section of an output decides about output settings
-            settings = output_settings.get(section, {})
-            result_settings[target].update(settings)
-            with target as out:
-                result[target].extend(
-                    func(
-                        target=out,
-                        width=settings.get("width", None),
-                        with_banner=settings.get("banner_top", True),
+            settings = output_settings[section]
+            if enabled is not None:
+                settings["enabled"] = enabled
+            if settings.get("enabled", True):
+                result_settings[target].update(settings)
+                with target as out:
+                    result[target].extend(
+                        func(
+                            target=out,
+                            width=settings.get("width", None),
+                            with_banner=settings.get("banner_top", True),
+                        )
                     )
-                )
+
+    history_handle_unchanged_contents()
 
     for target, res in result.items():
         settings = result_settings[target]
@@ -415,14 +744,31 @@ def context(subcontext=None) -> None:
             with target as out:
                 res.append(pwndbg.ui.banner("", target=out, width=settings.get("width", None)))
 
+    cmd_lines = 0
     for target, lines in result.items():
         with target as out:
             if result_settings[target].get("clearing", config_clear_screen) and lines:
                 clear_screen(out)
-            out.write("\n".join(lines))
-            if out is sys.stdout:
-                out.write("\n")
+            # The `if` here essentially checks whether the output is going to
+            # the window where the debugger prompt is. The prompt is
+            # always printed to stdout.
+            if isinstance(target, StdOutput):
+                # We always get a new line after the prompt, so the first line
+                # doesn't need to have a "\n" in front. Furthermore, we need
+                # to reserve a new line before the next prompt is printed.
+                out.writelines(line + "\n" for line in lines)
+            else:
+                # A "\n" after the last line would just take up space in this case.
+                # There will be a redundant preceding empty line when the first
+                # context is printed, but it doesn't really matter.
+                out.writelines("\n" + line for line in lines)
             out.flush()
+        # Assuming the command window (where the prompt is printed) is always stdout
+        if isinstance(target, StdOutput):
+            assert all("\n" not in line for line in lines)
+            cmd_lines += len(lines)
+
+    reserve_lines_maybe(cmd_lines)
 
 
 pwndbg.config.add_param(
@@ -449,10 +795,8 @@ def compact_regs(regs, width=None, target=sys.stdout):
     min_width = max(1, int(pwndbg.config.show_compact_regs_min_width))
     separation = max(1, int(pwndbg.config.show_compact_regs_separation))
 
-    if width is None:  # auto width. In case of stdout, it's better to use stdin (b/c GdbOutputFile)
-        _height, width = pwndbg.ui.get_window_size(
-            target=target if target != sys.stdout else sys.stdin
-        )
+    if width is None:
+        _height, width = pwndbg.ui.get_window_size(target)
 
     if columns > 0:
         # Adjust the minimum_width (column) according to the
@@ -507,6 +851,7 @@ def compact_regs(regs, width=None, target=sys.stdout):
     return result
 
 
+@serve_context_history
 def context_regs(target=sys.stdout, with_banner=True, width=None):
     regs = get_regs()
     if pwndbg.config.show_compact_regs:
@@ -520,15 +865,18 @@ def context_regs(target=sys.stdout, with_banner=True, width=None):
     return banner + regs if with_banner else regs
 
 
+@serve_context_history
 def context_heap_tracker(target=sys.stdout, with_banner=True, width=None):
-    if not pwndbg.gdblib.heap_tracking.is_enabled():
+    if not pwndbg.gdblib.ptmalloc2_tracking.is_enabled():
         return []
 
     banner = [pwndbg.ui.banner("heap tracker", target=target, width=width, extra="")]
 
-    if pwndbg.gdblib.heap_tracking.last_issue is not None:
-        info = [f"Detected the following potential issue: {pwndbg.gdblib.heap_tracking.last_issue}"]
-        pwndbg.gdblib.heap_tracking.last_issue = None
+    if pwndbg.gdblib.ptmalloc2_tracking.last_issue is not None:
+        info = [
+            f"Detected the following potential issue: {pwndbg.gdblib.ptmalloc2_tracking.last_issue}"
+        ]
+        pwndbg.gdblib.ptmalloc2_tracking.last_issue = None
     else:
         info = ["Nothing to report."]
 
@@ -557,28 +905,28 @@ def get_regs(regs: List[str] = None):
         regs = []
 
     if len(regs) == 0:
-        regs += pwndbg.gdblib.regs.gpr
+        regs += pwndbg.aglib.regs.gpr
 
-        regs.append(pwndbg.gdblib.regs.frame)
-        regs.append(pwndbg.gdblib.regs.stack)
+        regs.append(pwndbg.aglib.regs.frame)
+        regs.append(pwndbg.aglib.regs.stack)
 
         if pwndbg.config.show_retaddr_reg:
-            regs += pwndbg.gdblib.regs.retaddr
+            regs += pwndbg.aglib.regs.retaddr
 
-        regs.append(pwndbg.gdblib.regs.current.pc)
+        regs.append(pwndbg.aglib.regs.current.pc)
 
         if pwndbg.config.show_flags:
-            regs += pwndbg.gdblib.regs.flags.keys()
+            regs += pwndbg.aglib.regs.flags.keys()
 
-    changed = pwndbg.gdblib.regs.changed
+    changed = pwndbg.aglib.regs.changed
 
     for reg in regs:
         if reg is None:
             continue
 
-        value = pwndbg.gdblib.regs[reg]
+        value = pwndbg.aglib.regs[reg]
         if value is None:
-            print(message.warn("Unknown register: %r" % reg))
+            print(message.warn(f"Unknown register: {reg!r}"))
             continue
 
         # Make the register stand out and give a color if changed
@@ -587,17 +935,17 @@ def get_regs(regs: List[str] = None):
             regname = C.register_changed(regname)
 
         # Show a dot next to the register if it changed
-        change_marker = "%s" % C.config_register_changed_marker
+        change_marker = f"{C.config_register_changed_marker}"
         m = " " * len(change_marker) if reg not in changed else C.register_changed(change_marker)
 
         bit_flags = None
-        if reg in pwndbg.gdblib.regs.flags:
-            bit_flags = pwndbg.gdblib.regs.flags[reg]
-        elif reg in pwndbg.gdblib.regs.extra_flags:
-            bit_flags = pwndbg.gdblib.regs.extra_flags[reg]
+        if reg in pwndbg.aglib.regs.flags:
+            bit_flags = pwndbg.aglib.regs.flags[reg]
+        elif reg in pwndbg.aglib.regs.extra_flags:
+            bit_flags = pwndbg.aglib.regs.extra_flags[reg]
 
         if bit_flags:
-            desc = C.format_flags(value, bit_flags, pwndbg.gdblib.regs.last.get(reg, 0))
+            desc = C.format_flags(value, bit_flags, pwndbg.aglib.regs.last.get(reg, 0))
 
         else:
             desc = pwndbg.chain.format(value)
@@ -611,28 +959,38 @@ disasm_lines = pwndbg.config.add_param(
 )
 
 
-def context_disasm(target=sys.stdout, with_banner=True, width=None):
+def try_emulate_if_bug_disable(handler: Callable[[], T]) -> T:
     try:
-        flavor = gdb.execute("show disassembly-flavor", to_string=True).lower().split('"')[1]
-    except gdb.error as e:
-        if str(e).find("disassembly-flavor") > -1:
-            flavor = "intel"
-        else:
-            raise
+        return handler()
+    except U.UcError as e:
+        print(
+            message.warn(
+                f"Warning: Emulation context disabled due to a Unicorn error: \n{str(e)}\n"
+                "If you want to enable it again, use `set emulate on`."
+            )
+        )
+        pwndbg.config.emulate.value = "off"
+        return handler()
 
-    syntax = pwndbg.gdblib.disasm.CapstoneSyntax[flavor]
+
+@serve_context_history
+def context_disasm(target=sys.stdout, with_banner=True, width=None):
+    flavor = pwndbg.dbg.x86_disassembly_flavor()
+    syntax = pwndbg.aglib.disasm.CapstoneSyntax[flavor]
 
     # Get the Capstone object to set disassembly syntax
-    cs = next(iter(pwndbg.gdblib.disasm.get_disassembler_cached.cache.values()), None)
+    cs = next(iter(pwndbg.aglib.disasm.get_disassembler_cached.cache.values()), None)
 
     # The `None` case happens when the cache was not filled yet (see e.g. #881)
     if cs is not None and cs.syntax != syntax:
         pwndbg.lib.cache.clear_caches()
 
-    result = pwndbg.gdblib.nearpc.nearpc(
-        lines=disasm_lines // 2,
-        emulate=bool(not pwndbg.config.emulate == "off"),
-        use_cache=True,
+    result = try_emulate_if_bug_disable(
+        lambda: pwndbg.aglib.nearpc.nearpc(
+            lines=disasm_lines // 2,
+            emulate=bool(not pwndbg.config.emulate == "off"),
+            use_cache=True,
+        )
     )
 
     # Note: we must fetch emulate value again after disasm since
@@ -641,10 +999,10 @@ def context_disasm(target=sys.stdout, with_banner=True, width=None):
     thumb_mode_str = get_thumb_mode_string()
     if thumb_mode_str is not None:
         info = " / {} / {} mode / set emulate {}".format(
-            pwndbg.gdblib.arch.current, thumb_mode_str, pwndbg.config.emulate
+            pwndbg.aglib.arch.name, thumb_mode_str, pwndbg.config.emulate
         )
     else:
-        info = " / {} / set emulate {}".format(pwndbg.gdblib.arch.current, pwndbg.config.emulate)
+        info = " / {} / set emulate {}".format(pwndbg.aglib.arch.name, pwndbg.config.emulate)
     banner = [pwndbg.ui.banner("disasm", target=target, width=width, extra=info)]
 
     # If we didn't disassemble backward, try to make sure
@@ -684,23 +1042,22 @@ def get_filename_and_formatted_source():
     Returns formatted, lines limited and highlighted source as list
     or if it isn't there - an empty list
     """
-    sal = gdb.selected_frame().find_sal()  # gdb.Symtab_and_line
+    sal = pwndbg.dbg.selected_frame().sal()
 
     # Check if source code is available
-    if sal.symtab is None:
-        return "", []
+    if sal is None:
+        return "", [], 0
 
     # Get the full source code
-    closest_line = sal.line
-    filename = sal.symtab.fullname()
+    filename, closest_line = sal
 
     try:
         source = get_highlight_source(filename)
     except OSError:
-        return "", []
+        return "", [], closest_line
 
     if not source:
-        return "", []
+        return "", [], closest_line
 
     n = int(source_disasm_lines)
 
@@ -734,7 +1091,7 @@ def get_filename_and_formatted_source():
         )
         formatted_source.append(line)
 
-    return filename, formatted_source
+    return filename, formatted_source, closest_line
 
 
 should_decompile = pwndbg.config.add_param(
@@ -744,25 +1101,20 @@ should_decompile = pwndbg.config.add_param(
 )
 
 
+@serve_context_history
 def context_code(target=sys.stdout, with_banner=True, width=None):
-    filename, formatted_source = get_filename_and_formatted_source()
+    filename, formatted_source, line = get_filename_and_formatted_source()
 
     # Try getting source from files
     if formatted_source:
         bannerline = (
             [pwndbg.ui.banner("Source (code)", target=target, width=width)] if with_banner else []
         )
-        return (
-            bannerline
-            + ["In file: %s:%d" % (filename, gdb.selected_frame().find_sal().line)]
-            + formatted_source
-        )
+        return bannerline + [f"In file: {filename}:{line}"] + formatted_source
 
     if should_decompile:
         # Will be None if decompilation fails
-        code = pwndbg.integration.provider.decompile(
-            pwndbg.gdblib.regs.pc, int(source_disasm_lines)
-        )
+        code = pwndbg.integration.provider.decompile(pwndbg.aglib.regs.pc, int(source_disasm_lines))
 
         if code:
             bannerline = (
@@ -778,10 +1130,11 @@ stack_lines = pwndbg.config.add_param(
 )
 
 
+@serve_context_history
 def context_stack(target=sys.stdout, with_banner=True, width=None):
     result = [pwndbg.ui.banner("stack", target=target, width=width)] if with_banner else []
     telescope = pwndbg.commands.telescope.telescope(
-        pwndbg.gdblib.regs.sp, to_string=True, count=stack_lines
+        pwndbg.aglib.regs.sp, to_string=True, count=stack_lines
     )
     if telescope:
         result.extend(telescope)
@@ -796,21 +1149,22 @@ backtrace_frame_label = theme.add_param(
 )
 
 
+@serve_context_history
 def context_backtrace(with_banner=True, target=sys.stdout, width=None):
     result = []
 
     if with_banner:
         result.append(pwndbg.ui.banner("backtrace", target=target, width=width))
 
-    this_frame = gdb.selected_frame()
+    this_frame = pwndbg.dbg.selected_frame()
     newest_frame = this_frame
     oldest_frame = this_frame
 
     for i in range(backtrace_lines - 1):
         try:
-            candidate = oldest_frame.older()
-        # We catch gdb.error in case of a `gdb.error: PC not saved` case
-        except (gdb.MemoryError, gdb.error):
+            candidate = oldest_frame.parent()
+        # We catch an error in case of a `gdb.error: PC not saved` case
+        except pwndbg.dbg_mod.Error:
             break
 
         if not candidate:
@@ -818,35 +1172,34 @@ def context_backtrace(with_banner=True, target=sys.stdout, width=None):
         oldest_frame = candidate
 
     for i in range(backtrace_lines - 1):
-        candidate = newest_frame.newer()
+        candidate = newest_frame.child()
         if not candidate:
             break
         newest_frame = candidate
 
     frame = newest_frame
     i = 0
-    bt_prefix = "%s" % pwndbg.config.backtrace_prefix
+    bt_prefix = f"{pwndbg.config.backtrace_prefix}"
     while True:
         prefix = bt_prefix if frame == this_frame else " " * len(bt_prefix)
         prefix = f" {c.prefix(prefix)}"
         addrsz = c.address(pwndbg.ui.addrsz(frame.pc()))
-        symbol = c.symbol(pwndbg.gdblib.symbol.get(int(frame.pc())))
+        symbol = c.symbol(pwndbg.aglib.symbol.resolve_addr(int(frame.pc())))
         if symbol:
-            addrsz = addrsz + " " + symbol
-        line = map(str, (prefix, c.frame_label("%s%i" % (backtrace_frame_label, i)), addrsz))
-        line = " ".join(line)
-        result.append(line)
+            addrsz = f"{addrsz} {symbol}"
+        result.append(f"{prefix} {c.frame_label(f'{backtrace_frame_label}{i}')} {addrsz}")
 
         if frame == oldest_frame:
             break
 
-        frame = frame.older()
+        frame = frame.parent()
         i += 1
     return result
 
 
+@serve_context_history
 def context_args(with_banner=True, target=sys.stdout, width=None):
-    args = pwndbg.arguments.format_args(pwndbg.gdblib.disasm.one())
+    args = pwndbg.arguments.format_args(pwndbg.aglib.disasm.one())
 
     # early exit to skip section if no arg found
     if not args:
@@ -878,6 +1231,7 @@ def get_thread_status(thread):
         return "unknown"
 
 
+@serve_context_history
 def context_threads(with_banner=True, target=sys.stdout, width=None):
     try:
         original_thread = gdb.selected_thread()
@@ -907,24 +1261,33 @@ def context_threads(with_banner=True, target=sys.stdout, width=None):
     if len(displayed_threads) < 2:
         return []
 
-    out = [pwndbg.ui.banner(f"threads ({len(all_threads)} total)", target=target, width=width)]
+    out = (
+        [pwndbg.ui.banner(f"threads ({len(all_threads)} total)", target=target, width=width)]
+        if with_banner
+        else []
+    )
     max_name_length = 0
+    max_global_num_len = 0
 
     for thread in displayed_threads:
         name = thread.name or ""
         if len(name) > max_name_length:
             max_name_length = len(name)
+        if len(str(thread.global_num)) > max_global_num_len:
+            max_global_num_len = len(str(thread.global_num))
 
     for thread in filter(lambda t: t.is_valid(), displayed_threads):
         selected = " ►" if thread is original_thread else "  "
         name = thread.name if thread.name is not None else ""
-        padding = max_name_length - len(name)
+        name_padding = max_name_length - len(name)
+        global_num_padding = max(2, max_global_num_len - len(str(thread.global_num)))
         status = get_thread_status(thread)
 
         line = (
-            f" {selected} {thread.global_num}\t"
+            f" {selected} {thread.global_num} "
+            f"{' ' * global_num_padding}"
             f'"{pwndbg.color.cyan(name)}" '
-            f'{" " * padding}'
+            f'{" " * name_padding}'
             f"{status}: "
         )
 
@@ -933,7 +1296,7 @@ def context_threads(with_banner=True, target=sys.stdout, width=None):
             pc = gdb.selected_frame().pc()
 
             pc_colored = M.get(pc)
-            symbol = pwndbg.gdblib.symbol.get(pc)
+            symbol = pwndbg.aglib.symbol.resolve_addr(int(pc))
 
             line += f"{pc_colored}"
             if symbol:
@@ -963,7 +1326,7 @@ def save_signal(signal) -> None:
     if isinstance(signal, gdb.ExitedEvent):
         # Booooo old gdb
         if hasattr(signal, "exit_code"):
-            result.append(message.exit("Exited: %r" % signal.exit_code))
+            result.append(message.exit(f"Exited: {signal.exit_code}"))
 
     elif isinstance(signal, gdb.SignalEvent):
         msg = f"Program received signal {signal.stop_signal}"
@@ -973,11 +1336,11 @@ def save_signal(signal) -> None:
             # we can't access $_siginfo, so lets just show current pc
             # see also issue 476
             if _is_rr_present():
-                msg += " (current pc: %#x)" % pwndbg.gdblib.regs.pc
+                msg += f" (current pc: {pwndbg.aglib.regs.pc:#x})"
             else:
                 try:
                     si_addr = gdb.parse_and_eval("$_siginfo._sifields._sigfault.si_addr")
-                    msg += " (fault address %#x)" % int(si_addr or 0)
+                    msg += f" (fault address {int(si_addr):#x})"
                 except gdb.error:
                     pass
         result.append(message.signal(msg))
@@ -987,29 +1350,46 @@ def save_signal(signal) -> None:
             result.append(message.breakpoint(f"Breakpoint {(bkpt.location)}"))
 
 
-gdb.events.cont.connect(save_signal)
-gdb.events.stop.connect(save_signal)
-gdb.events.exited.connect(save_signal)
+if pwndbg.dbg.is_gdblib_available():
+    gdb.events.cont.connect(save_signal)
+    gdb.events.stop.connect(save_signal)
+    gdb.events.exited.connect(save_signal)
 
 
-def context_signal():
-    return last_signal
+@serve_context_history
+def context_last_signal(with_banner=True, target=sys.stdout, width=None):
+    if not last_signal:
+        return []
+
+    result = last_signal[::]
+    if with_banner:
+        result.insert(0, pwndbg.ui.banner("last signal", target=target, width=width))
+
+    return result
 
 
 context_sections = {
+    "a": context_args,
     "r": context_regs,
     "d": context_disasm,
-    "a": context_args,
-    "c": context_code,
     "s": context_stack,
     "b": context_backtrace,
-    "e": context_expressions,
-    "g": context_ghidra,
-    "h": context_heap_tracker,
-    "t": context_threads,
+    "c": context_code,
 }
 
 
+if pwndbg.dbg.is_gdblib_available():
+    # Add the sections that need gdblib.
+    context_sections = {
+        **context_sections,
+        "e": context_expressions,
+        "g": context_ghidra,
+        "h": context_heap_tracker,
+        "t": context_threads,
+        "l": context_last_signal,
+    }
+
+
 @pwndbg.lib.cache.cache_until("forever")
 def _is_rr_present() -> bool:
     """
