@@ -5,13 +5,14 @@ import argparse
 import pwndbg.chain
 import pwndbg.color as C
 import pwndbg.commands
+from pwndbg.commands import CommandCategory
 
 parser = argparse.ArgumentParser(description="Dumps the arguments of a va_list.")
 parser.add_argument("addr", type=int, help="Address of the va_list")
 parser.add_argument("count", type=int, nargs="?", default=8, help="Number of arguments to dump")
 
 
-@pwndbg.commands.ArgparsedCommand(parser)
+@pwndbg.commands.ArgparsedCommand(parser, category=CommandCategory.MISC)
 @pwndbg.commands.OnlyWhenRunning
 def valist(addr: int, count: int) -> None:
     # The `va_list` struct looks like this:
@@ -25,11 +26,11 @@ def valist(addr: int, count: int) -> None:
     # } va_list[1];
     # ```
 
-    gp_offset = pwndbg.gdblib.memory.u32(addr)
+    gp_offset = pwndbg.aglib.memory.u32(addr)
     gp_index = gp_offset / 8
 
-    overflow_arg_area = pwndbg.gdblib.memory.u64(addr + 8)
-    reg_save_area = pwndbg.gdblib.memory.u64(addr + 16)
+    overflow_arg_area = pwndbg.aglib.memory.u64(addr + 8)
+    reg_save_area = pwndbg.aglib.memory.u64(addr + 16)
 
     indent = " " * len("gp_offset => ")
     print(f"{C.blue('reg_save_area')}")
