@@ -20,7 +20,6 @@ from typing import List
 from typing import Tuple
 from typing import TypeVar
 
-import gdb
 import pygments
 import pygments.formatters
 import pygments.style
@@ -28,23 +27,24 @@ import pygments.token
 from typing_extensions import ParamSpec
 
 import pwndbg
+import pwndbg.aglib.arch
+import pwndbg.aglib.elf
+import pwndbg.aglib.proc
+import pwndbg.aglib.regs
+import pwndbg.aglib.symbol
 import pwndbg.color
 import pwndbg.color.context as context_color
 import pwndbg.decorators
-import pwndbg.gdblib.arch
-import pwndbg.gdblib.elf
-import pwndbg.gdblib.memory
-import pwndbg.gdblib.nearpc
-import pwndbg.gdblib.regs
-import pwndbg.gdblib.symbol
 import pwndbg.integration
 import pwndbg.lib.cache
 import pwndbg.lib.config
+from pwndbg.aglib.nearpc import c as nearpc_color
+from pwndbg.aglib.nearpc import ljust_padding
 from pwndbg.color import message
 from pwndbg.color import theme
+from pwndbg.dbg import BreakpointLocation
 from pwndbg.dbg import EventType
-from pwndbg.gdblib.nearpc import c as nearpc_color
-from pwndbg.gdblib.nearpc import ljust_padding
+from pwndbg.dbg import StopPoint
 from pwndbg.lib.functions import Argument
 from pwndbg.lib.functions import Function
 
@@ -153,13 +153,24 @@ def init_bn_rpc_client() -> None:
 
 
 def with_bn(fallback: K = None) -> Callable[[Callable[P, T]], Callable[P, T | K]]:
+    global _bn
+
     def decorator(func: Callable[P, T]) -> Callable[P, T | K]:
+        global _bn
+
         @functools.wraps(func)
         def wrapper(*args: P.args, **kwargs: P.kwargs) -> T | K:
+            global _bn
             if _bn is None:
                 init_bn_rpc_client()
-            if _bn is not None:
-                return func(*args, **kwargs)
+
+            try:
+                if _bn is not None:
+                    return func(*args, **kwargs)
+            except ConnectionRefusedError:
+                print(message.error("[!] Binary Ninja connection refused"))
+                _bn = None
+
             return fallback
 
         return wrapper
@@ -178,18 +189,18 @@ def can_connect() -> bool:
 
 
 def l2r(addr: int) -> int:
-    exe = pwndbg.gdblib.elf.exe()
+    exe = pwndbg.aglib.elf.exe()
     if not exe:
         raise Exception("Can't find EXE base")
-    result = (addr - pwndbg.gdblib.proc.binary_base_addr + base()) & pwndbg.gdblib.arch.ptrmask
+    result = (addr - pwndbg.aglib.proc.binary_base_addr + base()) & pwndbg.aglib.arch.ptrmask
     return result
 
 
 def r2l(addr: int) -> int:
-    exe = pwndbg.gdblib.elf.exe()
+    exe = pwndbg.aglib.elf.exe()
     if not exe:
         raise Exception("Can't find EXE base")
-    result = (addr - base() + pwndbg.gdblib.proc.binary_base_addr) & pwndbg.gdblib.arch.ptrmask
+    result = (addr - base() + pwndbg.aglib.proc.binary_base_addr) & pwndbg.aglib.arch.ptrmask
     return result
 
 
@@ -201,13 +212,15 @@ def base():
 @pwndbg.dbg.event_handler(EventType.STOP)
 @with_bn()
 def auto_update_pc() -> None:
-    pc = pwndbg.gdblib.regs.pc
+    if not pwndbg.aglib.proc.alive:
+        return
+    pc = pwndbg.aglib.regs.pc
     if bn_autosync.value:
         navigate_to(pc)
     _bn.update_pc_tag(l2r(pc))
 
 
-_managed_bps: Dict[int, gdb.Breakpoint] = {}
+_managed_bps: Dict[int, StopPoint] = {}
 
 
 @pwndbg.dbg.event_handler(EventType.START)
@@ -215,12 +228,17 @@ _managed_bps: Dict[int, gdb.Breakpoint] = {}
 @pwndbg.dbg.event_handler(EventType.CONTINUE)
 @with_bn()
 def auto_update_bp() -> None:
+    if not pwndbg.aglib.proc.alive:
+        return
     bps: List[int] = _bn.get_bp_tags()
     binja_bps = {r2l(addr) for addr in bps}
     for k in _managed_bps.keys() - binja_bps:
-        _managed_bps.pop(k).delete()
+        bp = _managed_bps.pop(k)
+        bp.remove()
+
+    inf = pwndbg.dbg.selected_inferior()
     for k in binja_bps - _managed_bps.keys():
-        bp = gdb.Breakpoint("*" + hex(k))
+        bp = inf.break_at(BreakpointLocation(k))
         _managed_bps[k] = bp
 
 
@@ -365,7 +383,8 @@ themes["light"] = LightTheme
 style = theme.add_param(
     "bn-decomp-style",
     "dark",
-    f"Decompilation highlight theme for Binary Ninja (valid values are {', '.join(themes.keys())})",
+    "decompilation highlight theme for Binary Ninja",
+    help_docstring=f"Valid values: {', '.join(themes.keys())})",
 )
 
 
@@ -452,7 +471,7 @@ class BinjaProvider(pwndbg.integration.IntegrationProvider):
         min_indents = None
         for addr, decomp_toks in sliced:
             addrs.append(hex(addr))
-            syms.append(f"<{pwndbg.gdblib.symbol.get(addr)}>")
+            syms.append(f"<{pwndbg.aglib.symbol.resolve_addr(addr)}>")
             indents = 0
             for _, ty in decomp_toks:
                 if ty == "IndentationToken":
@@ -506,29 +525,32 @@ class BinjaProvider(pwndbg.integration.IntegrationProvider):
     @with_bn()
     @pwndbg.lib.cache.cache_until("stop")
     def get_stack_var_name(self, addr: int) -> str | None:
-        cur = gdb.selected_frame()
+        cur = pwndbg.dbg.selected_frame()
         # there is no earlier frame so we give up
-        if addr < pwndbg.gdblib.regs.read_reg("sp", cur):
+        if addr < pwndbg.aglib.regs.read_reg("sp", frame=cur):
             return None
         newest = True
         # try to find the oldest frame that's earlier than the address
         while True:
-            upper = cur.older()
+            upper = cur.parent()
             if upper is None:
                 break
-            upper_sp = pwndbg.gdblib.regs.read_reg("sp", upper)
+
+            upper_sp = pwndbg.aglib.regs.read_reg("sp", frame=upper)
             if upper_sp > addr:
                 break
+
             cur = upper
             newest = False
+
         regs = [
             (name, val)
-            for name in pwndbg.gdblib.regs.common
-            if (val := pwndbg.gdblib.regs.read_reg(name, cur)) is not None
+            for name in pwndbg.aglib.regs.common
+            if (val := pwndbg.aglib.regs.read_reg(name, frame=cur)) is not None
         ]
         # put stack pointer and frame pointer at the front
         regs.sort(
-            key=lambda x: {pwndbg.gdblib.regs.stack: 0, pwndbg.gdblib.regs.frame: 1}.get(x[0], 2)
+            key=lambda x: {pwndbg.aglib.regs.stack: 0, pwndbg.aglib.regs.frame: 1}.get(x[0], 2)
         )
         ret: Tuple[int, str, int] | None = _bn.get_stack_var_name(l2r(int(cur.pc())), regs, addr)
         if ret is None:
