@@ -13,6 +13,8 @@ from typing import Set
 from typing import Tuple
 from typing import Union
 
+from pwndbg.lib.arch import PWNDBG_SUPPORTED_ARCHITECTURES_TYPE
+
 BitFlags = OrderedDict[str, Union[int, Tuple[int, int]]]
 
 
@@ -93,6 +95,10 @@ class RegisterSet:
 
         self.all = set(misc) | set(flags) | set(extra_flags) | set(self.retaddr) | set(self.common)
         self.all -= {None}
+        self.all |= {"pc", "sp"}
+
+    def __contains__(self, reg: str) -> bool:
+        return reg in self.all
 
     def __iter__(self) -> Iterator[str]:
         yield from self.all
@@ -193,14 +199,6 @@ arm = RegisterSet(
     retval="r0",
 )
 
-iwmmxt = RegisterSet(
-    retaddr=("lr",),
-    flags={"cpsr": arm_cpsr_flags},
-    gpr=("r0", "r1", "r2", "r3", "r4", "r5", "r6", "r7", "r8", "r9", "r10", "r11", "r12"),
-    args=("r0", "r1", "r2", "r3"),
-    retval="r0",
-)
-
 # ARM Cortex-M
 armcm = RegisterSet(
     retaddr=("lr",),
@@ -622,7 +620,55 @@ riscv = RegisterSet(
     retval="a0",
 )
 
-reg_sets = {
+# https://docs.kernel.org/arch/loongarch/introduction.html
+loongarch64 = RegisterSet(
+    pc="pc",
+    stack="sp",
+    frame="fp",
+    retaddr=("ra",),
+    gpr=(
+        "a0",
+        "a1",
+        "a2",
+        "a3",
+        "a4",
+        "a5",
+        "a6",
+        "a7",
+        "t0",
+        "t1",
+        "t2",
+        "t3",
+        "t4",
+        "t5",
+        "t6",
+        "t7",
+        "t8",
+        "s0",
+        "s1",
+        "s2",
+        "s3",
+        "s4",
+        "s5",
+        "s6",
+        "s7",
+        "s8",
+    ),
+    args=(
+        "a0",
+        "a1",
+        "a2",
+        "a3",
+        "a4",
+        "a5",
+        "a6",
+        "a7",
+    ),
+    # r21 stores "percpu base address", referred to as "u0" in the kernel
+    misc=("tp", "r21"),
+)
+
+reg_sets: Dict[PWNDBG_SUPPORTED_ARCHITECTURES_TYPE, RegisterSet] = {
     "i386": i386,
     "i8086": i386,
     "x86-64": amd64,
@@ -631,8 +677,8 @@ reg_sets = {
     "mips": mips,
     "sparc": sparc,
     "arm": arm,
-    "iwmmxt": iwmmxt,
     "armcm": armcm,
     "aarch64": aarch64,
     "powerpc": powerpc,
+    "loongarch64": loongarch64,
 }
